########## All borrowed and modified from JGL package.

###JGL_for_ATAC.utils
###penalty.as.matrix
###admm.iters_for_ATAC
###flsa.general (not used)
###flsa2
###dsgl
###soft
###admm.iters.unconnected
###crit
###gcrit

######################fast flsa (using flsa R package)
###flsa.general_matrix
###flsa.general_vector

penalty.as.matrix <- function(lambda,p,penalize.diagonal)
{
	# for matrix penalties:  check dim and symmetry:
	if(is.matrix(lambda))
	{
		if(sum(lambda!= t(lambda))>0) {stop("error: penalty matrix is not symmetric")}
		if(sum(abs(dim(lambda)-p))!=0 ) {stop("error: penalty matrix has wrong dimension")}
	}
	# for scalar penalties: convert to matrix form:
	if(length(lambda)==1) {lambda=matrix(lambda,p,p)}
	# apply the penalize.diagonal argument:
	if(!penalize.diagonal) {diag(lambda)=0}
	return(lambda)
}

### ADMM for FGL:
admm.iters_for_ATAC = function(Y,lambda1,lambda2,penalty="fused",rho=1,rho.increment=1,weights,penalize.diagonal,maxiter = 1000,tol=1e-5,warm=NULL)
{
	K = length(Y)
	p = dim(Y[[1]])[2]
	n=weights

	ns = c(); for(k in 1:K){ns[k] = dim(Y[[k]])[1]}
	S = list(); 
	#add 1e-5 at diag
	for(k in 1:K)
	{
	S[[k]] = cov(Y[[k]])*(ns[k]-1)/ns[k]
	diag(S[[k]]) <- diag(S[[k]]) + 1e-5
	}

	# initialize theta:
	theta = list()
	for(k in 1:K){theta[[k]] = diag(1/diag(S[[k]]))}
	# initialize Z:
	Z = list(); for(k in 1:K){Z[[k]]=matrix(0,p,p)}
	# initialize W:
	W = list();	for(k in 1:K) {W[[k]] = matrix(0,p,p) }

	# initialize lambdas:  (shouldn't need to do this if the function is called from the main wrapper function, JGL)
	lam1 = penalty.as.matrix(lambda1,p,penalize.diagonal=penalize.diagonal)
	if(penalty=="fused") {lam2 = penalty.as.matrix(lambda2,p,penalize.diagonal=TRUE)}
	if(penalty=="group") {lam2 = penalty.as.matrix(lambda2,p,penalize.diagonal=penalize.diagonal)}
	# iterations:
	iter=0
	diff_value = 10
	while((iter==0) || (iter<maxiter && diff_value > tol))
	{
		# reporting for test
		if(FALSE) {
			if(iter%%20==0)
		{
			print(paste("iter=",iter))
			if(penalty=="fused")
			{
				print(paste("crit of theta=",crit(theta,S,n=rep(1,K),lam1,lam2,penalize.diagonal=penalize.diagonal)))
				print(paste("crit of Z=",crit(Z,S,n=rep(1,K),lam1,lam2,penalize.diagonal=penalize.diagonal)))
			}
			if(penalty=="group"){print(paste("crit=",gcrit(theta,S,n=rep(1,K),lam1,lam2,penalize.diagonal=penalize.diagonal)))}
		}}

		# update theta:
		theta.prev = theta
		for(k in 1:K){
			edecomp = eigen(S[[k]] - rho*Z[[k]]/n[k] + rho*W[[k]]/n[k])
			D = edecomp$values
			V = edecomp$vectors
			D2 = n[k]/(2*rho) * ( -D + sqrt(D^2 + 4*rho/n[k]) )
			theta[[k]] = V %*% diag(D2) %*% t(V)
		}

		# update Z:
		# define A matrices:
		A = list()
		for(k in 1:K){ A[[k]] = theta[[k]] + W[[k]] }
		if(penalty=="fused")
		{
			# use flsa to minimize rho/2 ||Z-A||_F^2 + P(Z):
			if(K==2){Z = flsa2(A,rho,lam1,lam2,penalize.diagonal=TRUE)}
			if(K>2){Z = flsa.general(A,rho,lam1,lam2,penalize.diagonal=TRUE)}  # the option to not penalize the diagonal is exercised when we initialize the lambda matrices
		}
		if(penalty=="group")
		{
			#  minimize rho/2 ||Z-A||_F^2 + P(Z):
	   		Z = dsgl(A,rho,lam1,lam2,penalize.diagonal=TRUE)
		}

		# update the dual variable W:
		for(k in 1:K) {W[[k]] = W[[k]] + (theta[[k]]-Z[[k]])}

		# bookkeeping:
		iter = iter+1
	  	diff_value = 0
	       for(k in 1:K) {diff_value = diff_value + sum(abs(theta[[k]] - theta.prev[[k]])) / sum(abs(theta.prev[[k]]))}
		# increment rho by a constant factor:
		rho = rho*rho.increment
		
		#######diiff_value == NA,  print(error)
		if (is.na(diff_value)){
			return("ADMM not singular")
		}
	}
	diff = 0; for(k in 1:K){diff = diff + sum(abs(theta[[k]]-Z[[k]]))}
	out = list(theta=theta,Z=Z,diff=diff,iters=iter)
	return(out)
}

flsa.general <- function(A,L,lam1,lam2,penalize.diagonal)
{
	trueA = A
	if(is.matrix(A[[1]])) {p=dim(A[[1]])[1]}
	if(is.vector(A[[1]])) {p=length(A[[1]])}
	K = length(A)
	# results matrices:
	X = list()
	#for(k in 1:K) {X[[k]] = matrix(NA,p,p)} 
	for(k in 1:K) {X[[k]] = A[[1]]*NA} 
	if(is.matrix(A[[1]])) {fusions = array(FALSE,dim=c(K,K,p,p))}  
	if(is.vector(A[[1]])) {fusions = array(FALSE,dim=c(K,K,p,1))}

	# get starting newc: list of matrices.  newc[[k]][i,j] gives the (2*ordermats[[k]]-K-1) adjustment for lam2 at that element.  Basically, how many rank higher vs how many rank lower?
	newc = list()
	for(k in 1:K)
	{
		others = setdiff(1:K,k)
		others.smaller.k = 1:(k-1)
		newc[[k]] = A[[1]]*0               
		for(o in others) {newc[[k]] = newc[[k]] + (A[[o]]-A[[k]]< -1e-4) - (A[[o]]-A[[k]]>1e-4)} 
	}

	######### start the loop here:
	for(iter in 1:(K-1))
	{

	# create order matrices:
	ordermats = list()
	for(k in 1:K)
	{
		others = setdiff(1:K,k)
		others.smaller.k = 1:(k-1)
		ordermats[[k]] = A[[1]]*0   
		for(o in others) {ordermats[[k]] = ordermats[[k]] + (A[[k]]-A[[o]]>1e-4)} 
		# to deal with ties, also add a unit to ordermat[[k]] if a class with a lower k has a tie at element i,j:
		if(k>1)
		{
			for(o in others.smaller.k) {ordermats[[k]] = ordermats[[k]] + (abs(A[[o]]-A[[k]])<1e-4)} 
		}
		ordermats[[k]] = ordermats[[k]] + 1
	}

	# create beta.g matrices, holding the solution to Holger's "unconstrained problem" 
	#  (prending we're not constraining the order of the solution to match the order of the A matrices)
	betas.g = list()
	for(k in 1:K)
	{
		betas.g[[k]] = A[[k]] - lam2/L*newc[[k]]
	}

	# identify and fuse all elements for which the betas.g are out of order:
	new.ordermats = list()
	for(k in 1:K)
	{
		others = setdiff(1:K,k)
		others.smaller.k = 1:(k-1)
		new.ordermats[[k]] = A[[1]]*0   
		for(o in others) {new.ordermats[[k]] = new.ordermats[[k]] + (betas.g[[k]]-betas.g[[o]]>1e-4)}  
		# to deal with ties, also add a unit to ordermat[[k]] if a class with a lower k has a tie at element i,j:
		if(k>1)
		{
			for(o in others.smaller.k) {new.ordermats[[k]] = new.ordermats[[k]] + (abs(betas.g[[o]]-betas.g[[k]])<1e-4)} 
		}
		new.ordermats[[k]] = new.ordermats[[k]] + 1
	}

	# identify neighboring fusions:  "fusions": K x K x p x p array: K x K matrices, T/F for fusions 
	for(k in 1:K){
	for(kp in 1:K){
		#given k,kp, declare a fusion when their ordermats entries are adjacent, and their new.ordermats entries have the opposite direction:
		fusions[k,kp,,] = fusions[k,kp,,]+           
					((ordermats[[k]]-1==ordermats[[kp]])&(new.ordermats[[k]]<new.ordermats[[kp]]))+
					((ordermats[[k]]+1==ordermats[[kp]])&(new.ordermats[[k]]>new.ordermats[[kp]]))+
					(abs(A[[k]]-A[[kp]])<1e-4)
					#(existing fusions, neighboring fusions, and ties)
		fusions = (fusions>0)*1
	}}
								

	# now we've noted fusions between all entries with adjacent ordermats entries and reversed new.ordermats entries
	# next: extend fusions to non-adjecent entries: if a-b and b-c, then connect a-c:
	for(k in 1:K){
	for(kp in 1:K){
		others = setdiff(1:K,c(k,kp))
		for(o in others)
		{
			#identify elements in o which are fused with the same element in both k and kp, then add them to the list of k-kp fusions:
			bothfused = fusions[k,o,,] & fusions[kp,o,,]    
			fusions[k,kp,,] = fusions[k,kp,,] | bothfused       
		}
	}}

	# now recalculate A with the new fused entries:
	# to recalculate A, for each non-zero entry, identify the classes k with which it must be fused, and get their average:
	for(k in 1:K)
	{
		others = setdiff(1:K,k)
		#fusemean and denom: the mean value of all the trueA to be fused, and the number of values to be fused:
		fusemean = trueA[[k]]
		denom = A[[1]]*0+1   
		for(o in others)
		{
			fusemean = fusemean+fusions[k,o,,]*trueA[[o]]  #add the values of the elements which must be fused to fusemean   
			denom = denom+fusions[k,o,,]     
		}	
		# now redefine A[[k]]: unchanged from trueA if there's no fusion, and the mean of the fused elements when there is fusion:
		A[[k]] = fusemean/denom
	}

	#newc: list of matrices.  newc[[k]][i,j] gives the (2*ordermats[[k]]-K-1) adjustment for lam2 at that element.  Basically, how many rank higher vs how many rank lower?
	newc = list()
	for(k in 1:K)
	{
		others = setdiff(1:K,k)
		others.smaller.k = 1:(k-1)
		newc[[k]] = A[[1]]*0   
		for(o in others) {newc[[k]] = newc[[k]] + (A[[o]]-A[[k]]< -1e-4) - (A[[o]]-A[[k]]>1e-4)}  
	}

	} #end loop here

	# final version of betas.g:
	for(k in 1:K)
	{
		betas.g[[k]] = A[[k]] - lam2/L*newc[[k]]
	}
	# now soft-threshold the solution matices:
	for(k in 1:K)
	{
		X[[k]] = soft(betas.g[[k]],lam=lam1/L,penalize.diagonal=penalize.diagonal)
	}
	return(X)
}


flsa2 <- function(A,L,lam1,lam2,penalize.diagonal)  
{
	# 1st apply fused penalty:
	S1 = abs(A[[1]]-A[[2]])<=2*lam2/L
	X1 = (A[[1]]+A[[2]])/2
	Y1 = X1

	S2 = (A[[1]] > A[[2]]+2*lam2/L)
	X2 = A[[1]] - lam2/L
	Y2 = A[[2]] + lam2/L

	S3 = (A[[2]] > A[[1]]+2*lam2/L)
	X3 = A[[1]] + lam2/L
	Y3 = A[[2]] - lam2/L

	X = soft(a = S1*X1 + S2*X2 + S3*X3, lam = lam1/L, penalize.diagonal=penalize.diagonal)
	Y = soft(a = S1*Y1 + S2*Y2 + S3*Y3, lam = lam1/L, penalize.diagonal=penalize.diagonal)

	return(list(X,Y))
}


dsgl <- function(A,L,lam1,lam2,penalize.diagonal)
{
	lam1 = lam1*1/L
	lam2 = lam2*1/L

  if(is.matrix(A[[1]])) {p=dim(A[[1]])[1]}
  if(is.vector(A[[1]])) {p=length(A[[1]])}
	K=length(A)
	softA = A
	for(k in 1:K) {softA[[k]] = soft(A[[k]],lam1,penalize.diagonal=penalize.diagonal) }   #if penalize.diagonal=FALSE was used in ggl(), then this will not penalize the diagonal.
	normsoftA = A[[1]]*0
	for(k in 1:K) {normsoftA = normsoftA + (softA[[k]])^2}

	normsoftA = sqrt(normsoftA)

	notshrunk = (normsoftA>lam2)*1
	# reset 0 elements of normsoftA to 1 so we don't get NAs later. 
	normsoftA = normsoftA + (1-notshrunk)

	out = A
	for(k in 1:K)
	{
		out[[k]] = softA[[k]]*(1-lam2/normsoftA)
		out[[k]] = out[[k]]*notshrunk
	}
	return(out)
}


soft <- function(a,lam,penalize.diagonal)
{ # if last argument is FALSE, soft-threshold a matrix but don't penalize the diagonal
  out <- sign(a)*pmax(0, abs(a)-lam)
  if(!penalize.diagonal) diag(out) <- diag(a)
  return(out)
}


admm.iters.unconnected = function(Y,lambda1,lambda2,penalty="fused",rho=1,rho.increment=1,weights,maxiter = 1000,tol=1e-5,warm=NULL)
{
	K = length(Y)
	for(k in 1:K){Y[[k]]=as.matrix(Y[[k]])}
	p = dim(Y[[1]])[2]
	n=weights
  
 	# define S vectors: empirical covs of each element
	S = list()
	for(k in 1:K)
	{
  	ntemp=dim(Y[[k]])[1]
  	S[[k]]=rep(0,p)
  	for(j in 1:p)
  	{
  		S[[k]][j] = var(Y[[k]][,j])*(ntemp-1)/ntemp + 1e-4
  	}
	}

	# initialize theta:
	theta = list()
	for(k in 1:K){theta[[k]] = 1/S[[k]]}
	# initialize Z:
	Z = list(); for(k in 1:K){Z[[k]] = rep(0,p)}
	# initialize W:
	W = list();	for(k in 1:K){W[[k]] = rep(0,p)}

	# initialize lambdas:
	lam1 = lambda1
	lam2 = lambda2

	# iterations:
	iter=0
	diff_value = 10
	while((iter==0) || (iter<maxiter && diff_value > tol))
	{
	# update theta to minimize -logdet(theta) + <S,theta> + rho/2||theta - Z + W ||^2_2:
	theta.prev = theta
	for(k in 1:K)
	{
		B = n[k]*S[[k]] - rho*(Z[[k]] - W[[k]])  
		theta[[k]] = 1/(2*rho) * ( -B + sqrt(B^2 + 4*rho*n[k]) )  
#		B = S[[k]] - rho/n[k]*(Z[[k]] - W[[k]])  
#		theta[[k]] = n[k]/(2*rho) * ( -B + sqrt(B^2 + 4*rho/n[k]) )  # written as in our paper
	}

	# update Z:
	# define A matrices:
	A = list()
	for(k in 1:K){ A[[k]] = theta[[k]] + W[[k]] }
	if(penalty=="fused")
	{
		# use flsa to minimize rho/2 ||Z-A||_F^2 + P(Z):
		if(K==2){Z = flsa2(A,rho,lam1,lam2,penalize.diagonal=TRUE)}
		if(K>2){Z = flsa.general(A,rho,lam1,lam2,penalize.diagonal=TRUE)}
	}
	if(penalty=="group")
	{
		 # minimize rho/2 ||Z-A||_F^2 + P(Z):
		Z = dsgl(A,rho,lam1,lam2,penalize.diagonal=TRUE)
	}

	# update the dual variable W:
	for(k in 1:K){W[[k]] = W[[k]] + (theta[[k]]-Z[[k]])}
	
	# bookkeeping:
	iter = iter+1
  	diff_value = 0
	for(k in 1:K) {diff_value = diff_value + sum(abs(theta[[k]] - theta.prev[[k]])) / sum(abs(theta.prev[[k]]))}
	# increment rho by a constant factor:
	rho = rho * rho.increment
	}
	diff = 0; for(k in 1:K){diff = diff + sum(abs(theta[[k]]-Z[[k]]))}
	out = list(theta=theta,Z=Z,diff=diff,iters=iter)
	return(out)
}

crit <- function(theta, S,n, lam1, lam2, penalize.diagonal)  
{
	p = dim(S[[1]])[1]
	K = length(S)
	lam1 = penalty.as.matrix(lam1,p,penalize.diagonal=penalize.diagonal)
	lam2 = penalty.as.matrix(lam2,p,penalize.diagonal=TRUE)	
	crit = 0
	for(k in 1:length(theta))
	{
		# add log det that was entered as an argument, or else calculate it
		crit = crit+n[k]*log(det(theta[[k]]))-n[k]*sum(S[[k]]*theta[[k]])-sum(lam1*abs(theta[[k]])) 
		for(kp in k:length(theta))
		{
			crit = crit - sum(lam2*abs(theta[[k]]-theta[[kp]]))
		}
	}
	return(crit)
}

gcrit <- function(theta, S,n, lam1, lam2, penalize.diagonal)  
{
	p = dim(S[[1]])[1]
	K = length(S)
	lam1 = penalty.as.matrix(lam1,p,penalize.diagonal=penalize.diagonal)
	lam2 = penalty.as.matrix(lam2,p,penalize.diagonal=penalize.diagonal)	
	crit = 0
	sumsquared = matrix(0,p,p)
	for(k in 1:length(theta))
	{
		# add log det that was entered as an argument, or else calculate it
		crit = crit+n[k]*log(det(theta[[k]]))-n[k]*sum(S[[k]]*theta[[k]])-sum(lam1*abs(theta[[k]])) 
		# add to term that will become group L2 norm:
		sumsquared = sumsquared + theta[[k]]^2
	}
	crit = crit - sum(lam2*sqrt(sumsquared))
	return(crit)
}


flsa.general_matrix <- function(A,L,lam1,lam2)
{
	K <- length(A)
	Z <- list()
	for (temp_k in 1:K) {Z[[temp_k]] <- A[[temp_k]] * NA}
	#lam1_flsa <- 2 * lam1/L
	#lam2_flsa <- 2 * lam2/L
	lam1_flsa <- lam1/L
	lam2_flsa <- lam2/L
	lam1_temp <- lam1_flsa
	lam2_temp <- lam2_flsa
	if(is.matrix(A[[1]])) {p=dim(A[[1]])[1]}
	#if(is.vector(A[[1]])) {p=length(A[[1]])}
	for (i1 in 1:p) {
		for (i2 in 1:p) {
			temp_A = c()
			for (temp_k in 1:K) { 
				temp_A = c(temp_A, A[[temp_k]][i1,i2])
			}
			if(length(lam1_flsa)>1) {lam1_temp <- lam1_flsa[i1,i2]}
			if(length(lam2_flsa)>1) {lam2_temp <- lam2_flsa[i1,i2]}
			res_Z = flsa::flsa(temp_A,lambda1=lam1_temp, lambda2=lam2_temp)
			if (length(dim(res_Z)) == 3) { 
				res_Z <- res_Z[1,,]
			} 
			else {
				res_Z <- res_Z[1,]
			}
			for (temp_k in 1:K) { 
				Z[[temp_k]][i1,i2] = res_Z[temp_k]
			}
		}
	}
	return(Z)
}



flsa.general_vector <- function(A,L,lam1,lam2)
{
	K <- length(A)
	Z <- list()
	for (temp_k in 1:K) {Z[[temp_k]] <- A[[temp_k]] * NA}
	#lam1_flsa <- 2 * lam1/L
	#lam2_flsa <- 2 * lam2/L
	lam1_flsa <- lam1/L
	lam2_flsa <- lam2/L
	lam1_temp <- lam1_flsa
	lam2_temp <- lam2_flsa
	#if(is.matrix(A[[1]])) {p=dim(A[[1]])[1]}
	if(is.vector(A[[1]])) {p=length(A[[1]])}
	for (i1 in 1:p) {
		temp_A = c()
		for (temp_k in 1:K) {
			temp_A = c(temp_A, A[[temp_k]][i1])
		}
		if(length(lam1_flsa)>1) {lam1_temp <- lam1_flsa[i1]}
		if(length(lam2_flsa)>1) {lam2_temp <- lam2_flsa[i1]}
		res_Z = flsa::flsa(temp_A,lambda1=lam1_temp, lambda2=lam2_temp)
		if (length(dim(res_Z)) == 3) {
			res_Z <- res_Z[1,,]
		} 
		else {
			res_Z <- res_Z[1,]
		}
		for (temp_k in 1:K) { 
			Z[[temp_k]][i1] = res_Z[temp_k]
		}
	}
	return(Z)
}
