#' @export
#'
find_JRIM_parameter <- function(exprs_list, 
                                dist_mat, 
                                w_fix, 
                                maxit, 
                                s, 
                                distance_constraint, 
                                parameter_convergence, 
                                fixed="w2",
                                penalize.diagonal=FALSE, 
                                weights="equal",
                                verbose=TRUE,
                                sparsity_constrain=0.1,
                                long_dist_constrain=0.05,
                                similarity_constrain=0.6)
{
  if (sum(dist_mat > distance_constraint)/2 < 1) {
    return("No long edges")
  }
 
  found <- FALSE
  K <- length(exprs_list)

  starting_max = 0.5

  if (fixed=="w2") {
    starting_max <- 0.5
  }
  if (fixed=="w1") {
    starting_max <- 0.99
  }

  
  distance_parameter <- starting_max
  distance_parameter_max <- starting_max
  distance_parameter_min <- 0

  it <- 0
  if (fixed == "w2"){
    while(found != TRUE & it < maxit) {
      if (verbose){
        message("it=",it," ,parameter=",distance_parameter)
      } 

      lambda1_temp = distance_parameter - distance_parameter * w_fix
      lambda2_temp <- sqrt(2) * distance_parameter * w_fix

      rho_mat <- get_rho_mat(dist_mat, lambda1_temp, s)

      jgl.results <- JGL_for_ATAC(Y=exprs_list,penalty="group",lambda1=rho_mat,lambda2=lambda2_temp,
                    weights=weights,penalize.diagonal=penalize.diagonal,tol=1e-3,
                    maxiter=300,return.whole.theta=TRUE)

      big_entries <- sum(dist_mat > distance_constraint) * K
      Total_entries <- (nrow(jgl.results$theta[[1]])^2) * K
      non_zero_big_entries <- 0 
      non_zero_entries <- 0 
      for (i in 1:K) {
        non_zero_big_entries <- non_zero_big_entries + sum(jgl.results$theta[[i]][dist_mat > distance_constraint] != 0)
        non_zero_entries <- non_zero_entries + sum(jgl.results$theta[[i]] != 0)
      }

      if (((non_zero_big_entries/big_entries) > long_dist_constrain) | (non_zero_entries/Total_entries > sparsity_constrain)) {
        sparsity_limit <- FALSE
      } else {
        sparsity_limit <- TRUE
      }
      if (sparsity_limit != TRUE | (distance_parameter == 0)) {
        distance_parameter_min <- distance_parameter
      } else {
        distance_parameter_max <- distance_parameter
      }
      new_distance_parameter <- (distance_parameter_min + distance_parameter_max)/2
      if(new_distance_parameter == starting_max) {
        new_distance_parameter <- 2 * starting_max
        starting_max <- new_distance_parameter
      }
      if (parameter_convergence > abs(distance_parameter-new_distance_parameter)) {
        found <- TRUE
      } else {
        distance_parameter <- new_distance_parameter
      }
      it <- it + 1
      if (verbose) {
        message("---Big:",non_zero_big_entries/big_entries,",sparsity:",non_zero_entries/Total_entries)
      }
    }
  } 
  else {
    if (fixed=="w1") {
      while(found != TRUE & it < maxit) {
        if (verbose) {
          message("it=",it," ,parameter=",distance_parameter)
        }

        lambda1_temp <- w_fix - w_fix * distance_parameter
        lambda2_temp <- sqrt(2) * w_fix * distance_parameter


        rho_mat <- get_rho_mat(dist_mat, lambda1_temp, s)
        jgl.results <- JGL_for_ATAC(Y=exprs_list,penalty="group",lambda1=rho_mat,lambda2=lambda2_temp,
                      weights=weights,penalize.diagonal=penalize.diagonal,tol=1e-3,
                      maxiter=300,return.whole.theta=TRUE)


        common2_entries <- get_common2_entries(jgl.results$theta)/(K*(K-1)/2)
        non_zero_entries <- 0 
        for (i in 1:K) {
          non_zero_entries <- non_zero_entries + sum(jgl.results$theta[[i]] != 0)
        }
        non_zero_entries <- non_zero_entries/K
        if (common2_entries/non_zero_entries < similarity_constrain) {
          similarity <- FALSE
        } 
        else {
          similarity <- TRUE
        }
        if (similarity != TRUE | (distance_parameter == 0)) {
          distance_parameter_min <- distance_parameter
        } 
        else {
          distance_parameter_max <- distance_parameter
        }
        new_distance_parameter <- (distance_parameter_min + distance_parameter_max)/2
        if(new_distance_parameter == starting_max) {
          new_distance_parameter <- 2 * starting_max
          starting_max <- new_distance_parameter
        }
        if (parameter_convergence > abs(distance_parameter-new_distance_parameter)) {
          found <- TRUE
        } 
        else {
          distance_parameter <- new_distance_parameter
        }
        if (distance_parameter > 1){return("similarity FALSE")}
        it <- it + 1
        if (verbose) {
          message("---similarity:",common2_entries/non_zero_entries)
        }
      }
    } 
    else {
      return("fixed should be lambda1 or lambda2")
    }
  }
  if (maxit == it) warning("maximum iterations hit")
  return(distance_parameter)
}

#get common pairs in pairs tissues
get_common2_entries <- function(M_list)
{
  K = length(M_list)
  common2_matrix = (M_list[[1]] != 0)
  for (i in 2:K) {
    common2_matrix = common2_matrix + (M_list[[i]] != 0)
  }
  common2_matrix = as.matrix(common2_matrix)
  common2 = sum(common2_matrix * (common2_matrix-1)/2)
  return(common2)
}