########## All borrowed and modified from Cicero package.

##########generate_JRIM_models.utils
###1. generate_windows
###2. get_genomic_range
###3. calc_dist_matrix
###4. get_rho_mat

########################
###1 generate_windows
generate_windows <- function(window, genomic_coords) 
{
  if(!class(genomic_coords) == "data.frame") {
    chr_maxes <- read.table(genomic_coords)
  } 
  else {
    chr_maxes <- genomic_coords
  }
  names(chr_maxes) <- c("V1", "V2")
  win_ranges <- plyr::ddply(chr_maxes, plyr::.(V1), function(x) {
    r <- seq(from = 1, to = x$V2[1], by = window/2)
    l <- r + window - 1
    data.frame(start = r, end = l)
    })
  gr <- GenomicRanges::GRanges(win_ranges$V1,ranges=IRanges::IRanges(win_ranges$start, win_ranges$end))
  return(gr)
}


########################
###1.2 get_genomic_range
get_genomic_range <- function(grs, cds, win) 
{
  end1 <- as.numeric(as.character(GenomicRanges::end(grs[win])))
  end2 <- as.numeric(as.character(GenomicRanges::start(grs[win])))
  win_range <- cds[(fData(cds)$bp1 < end1 & fData(cds)$bp1 > end2) |
                    (fData(cds)$bp2 < end1 & fData(cds)$bp2 > end2), ]
  win_range <-
    win_range[as.character(fData(win_range)$chr) ==
                gsub("chr", "", as.character(GenomicRanges::seqnames(grs[win]))),]
  fData(win_range)$mean_bp <-
    (as.numeric(as.character(fData(win_range)$bp1)) +
       as.numeric(as.character(fData(win_range)$bp2)))/2
  return(win_range)
}	


########################
###3. calc_dist_matrix
calc_dist_matrix <- function(gene_range) 
{
  dist_mat <- as.matrix(dist(fData(gene_range)$mean_bp))
  row.names(dist_mat) <- colnames(dist_mat) <- row.names(fData(gene_range))
  return(dist_mat)
}



########################
###4. get_rho_mat
get_rho_mat <- function(dist_matrix, distance_parameter, s) 
{
  xmin <- 1000
  out <- (1-(xmin/dist_matrix)^s) * distance_parameter
  out[!is.finite(out)] <- 0
  out[out < 0] <- 0
  return(out)
}