########## All borrowed and modified from Cicero package.

###is_color
###is_chr
###generate_plotting_subset
###make_peak_track
###signif_up
###plotBedpe
###plotpair
###make_gene_model_track



##############
is_color <- function(x, df=NULL) {
  if (!is.null(df)) {
    if (all(vapply(x, function(y) y %in% names(df), TRUE))) return(TRUE)
    tryCatch(is.matrix(col2rgb(x)), error = function(e) FALSE)
  } else {
    tryCatch(is.matrix(col2rgb(x)), error = function(e) FALSE)
  }
}

##############
is_chr <- function(x) {
  assertthat::assert_that(is.character(x))
  grepl("chr", x)
}

##############
generate_plotting_subset <- function(connections, chr, minbp, maxbp) {
  connections$Peak1 <- as.character(connections$Peak1)
  connections$Peak2 <- as.character(connections$Peak2)

  pcolor_map <- data.frame(Peak1 = connections$Peak1,
                           peak_color = connections$peak_color)
  pcolor_map <- pcolor_map[!duplicated(pcolor_map),]
  connections$peak_color <- NULL

  if(sum(!c("chr_1", "chr_2", "bp1_1", "bp2_1", "bp2_1", "bp2_2") %in%
         names(connections)) != 0 ) {
    connections$chr <- NULL
    connections$bp1 <- NULL
    connections$bp2 <- NULL
    connections$chr_2 <- NULL
    connections$bp1_2 <- NULL
    connections$bp2_2 <- NULL

    connections <- cbind(connections, df_for_coords(connections$Peak1)[,c(1, 2, 3)])
    cons2 <- df_for_coords(connections$Peak2) #slow
    cons2$Peak <- NULL
    names(cons2) <- c("chr_2", "bp1_2", "bp2_2")
    connections <- cbind(connections, cons2) #slow
  } else {
    if(!grepl("chr", connections$chr_1[1])) {
      connections$chr_1 <- paste0("chr", connections$chr_1)
    }
    if(!grepl("chr", connections$chr_2[1])) {
      connections$chr_2 <- paste0("chr", connections$chr_2)
    }
    names(connections)[names(connections) == "chr_1"] <- "chr"
    names(connections)[names(connections) == "bp1_1"] <- "bp1"
    names(connections)[names(connections) == "bp2_1"] <- "bp2"
  }

  sub <- connections[connections$chr_2 == chr & connections$bp1 <= maxbp &
                       connections$bp2 <= maxbp & connections$bp1 >= minbp &
                       connections$bp2 >= minbp & connections$bp1_2 <= maxbp &
                       connections$bp2_2 <= maxbp & connections$bp1_2 >= minbp &
                       connections$bp2_2 >= minbp,]

  sub2 <- sub
  names(sub2)[unlist(lapply(c("Peak1", "Peak2", "coaccess", "chr",
                       "bp1", "bp2", "chr_2", "bp1_2", "bp2_2"),
    function(x) which(names(sub2) == x)))] <- c("Peak2", "Peak1", "coaccess",
                                               "chr_2", "bp1_2", "bp2_2",
                                               "chr", "bp1", "bp2")

  sub <- rbind(sub, sub2)
  sub <- sub[!duplicated(sub),]

  sub <- merge(sub, pcolor_map, all.x = TRUE)
  sub$peak_color <- as.character(sub$peak_color)
  sub$peak_color[is.na(sub$peak_color)] <- "black"

  return(sub)
}

##############
make_peak_track <- function(df) {
  df <- df[!duplicated(df[,c("chr", "bp1", "bp2", "peak_color")]),]

  if (sum(duplicated(df[,c("chr", "bp1", "bp2")])) > 0)
    stop(paste("Multiple peak colors correspond to a single peak. Be sure that",
               "your peak_color column name assigns colors for Peak1 only",
               collapse = " "))
  gr <-  GenomicRanges::GRanges(as.character(df$chr),
                 IRanges::IRanges(as.numeric(as.character(df$bp1)),
                         as.numeric(as.character(df$bp2))))

  anntrack <- Gviz::AnnotationTrack(gr, name="Peaks", fill = df$peak_color,
                                    lwd = .0000001, fontsize.group=6,
                                    fontsize=6, cex.feature = 0.5)

  anntrack
}

##############
signif_up <- function(x) {
  num_string <- format(x, scientific=TRUE)

  n <- strsplit(num_string, "e")
  n1 <- vapply(n, function(x) as.numeric(x[1]), .1)
  n2 <- vapply(n, function(x) as.numeric(x[2]), .1)

  ceiling(n1) * 10^(n2)
}

##############
plotBedpe <- function(bedpedata,
                      chrom,
                      chromstart,
                      chromend,
                      ymax,
                      coaccess_cutoff,
                      width,
                      alpha_by_coaccess,
                      color_names = NULL)
{ ###### All borrowed and modified from Sushi package.

  if (nrow(bedpedata) == 0) {
    warning("Nothing to plot")
    return()
  }

  bedpedata  <- bedpedata[,c("chrom1","start1","stop1","chrom2","start2",
                             "stop2", "height", "width", "color")]

  # normalize height
  maxheight <- ymax

  bedpedata$alpha <- .6
  if(alpha_by_coaccess) {
    bedpedata$alpha <- (bedpedata$height-coaccess_cutoff)/maxheight
  }
  bedpedata$height <- bedpedata$height/maxheight
  # remove any rows with 0 height
  bedpedata <- bedpedata[abs(bedpedata$height) > 0,]

  # reclass data
  for(i in c("start1", "stop1", "start2", "stop2")) {
    bedpedata[,i] <- as.numeric(as.character((bedpedata[,i])))
  }

  # add position columns
  bedpedata$pos1 = apply(bedpedata[,c("start1","stop1")],1,mean)
  bedpedata$pos2 = apply(bedpedata[,c("start2","stop2")],1,mean)

  totalrange <- as.numeric(as.character(chromend)) -
    as.numeric(as.character(chromstart))
  if (nrow(bedpedata) == 0) warning("Nothing to plot")

  #legFactors <- sort(names(which(apply(legInfo, 2, any))))
  #boxSize <-  if(length(setdiff(legFactors, c("col", "cex")))==0) 0.1 else 0.3
  #pcols <- Gviz:::.getPlottingFeatures(GdObject)

  if (!is.null(color_names)) {
    boxSize <- .3
    spacing <- 0.2
    vspace <- .05
    for (i in seq_len(length(color_names))) {
      grid::grid.lines(unit(c(spacing,spacing + boxSize), "inches"),
                       c(1 - vspace*i, 1 - vspace*i),
                       gp=grid::gpar(col=color_names[i], lwd=width))
      grid::grid.text(x=unit(.1 + (boxSize + spacing), "inches"),
                      y=1 - vspace*i, just=c(0, 0.5),
                      label=names(color_names)[i])
    }
  }

  # plot the data
  grid::grid.function(function(x) list(x=x, y=(coaccess_cutoff/(ymax))),
                      gp=grid::gpar(col="black", lty="dashed", lwd=width)) #
  for (row in (seq_len(nrow(bedpedata)))) {
    x1     = bedpedata$pos1[row]
    x2     = bedpedata$pos2[row]
    height = bedpedata$height[row]
    width  = bedpedata$width[row]
    color  = bedpedata$color[row]
    alpha  = bedpedata$alpha[row]
    plotpair(x1,x2,height,totalrange,width,color, chromstart, alpha)
  }
}

##############
plotpair <- function(start, end, height, totalrange,
                     width, color, chromstart, alpha) {
  #scale values for plotting
  x1 = (min(start,end) - as.numeric(as.character(chromstart)))/totalrange
  x2 = (max(start,end) - as.numeric(as.character(chromstart)))/totalrange
  hx1 <- (x1 + x2)/2
  hy1 <- height/.725

  grid::grid.bezier(x = c(x1, hx1, hx1, x2), y = c(0, hy1, hy1, 0),
                    default.units = "npc",
                    gp=grid::gpar(col=color, lwd=width,
                                  alpha = (alpha*.9 + .1)))
}

##############
make_gene_model_track <- function(txdb,
                                  chr,
                                  collapseTranscripts,
                                  gene_model_color, shape) {
  if(nrow(txdb) != 0) {
    txdb <- txdb[txdb$chromosome == chr,]
    txdb$transcript <- as.character(txdb$transcript)
    grtrack <- Gviz::GeneRegionTrack(txdb, chromosome = chr, geneSymbols=TRUE,
                                     name = "Gene Model", fill=gene_model_color,
                                     col= gene_model_color,  fontcolor="black",
                                     fontcolor.group="black", fontsize.group=6,
                                     fontsize=6,
                                     collapseTranscripts = collapseTranscripts,
                                     shape=shape)
  } else {
    grtrack <- GeneRegionTrack()
  }
  grtrack
}