% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_JRIM_parameters.R
\name{estimate_JRIM_parameters}
\alias{estimate_JRIM_parameters}
\title{Calculate scaling parameters}
\usage{
estimate_JRIM_parameters(cds_list, s = 0.75, maxit = 50,
  weights = "equal", w2_initial = 0.1, window = 5e+05,
  sample_num = 50, max_elements = 200, penalize.diagonal = FALSE,
  distance_constraint = 250000, w1_convergence = 1e-04,
  w2_convergence = 1e-04, verbose = TRUE,
  genomic_coords = cicero::human.hg19.genome, sparsity_constrain = 0.1,
  long_dist_constrain = 0.05, similarity_constrain = 0.6)
}
\arguments{
\item{cds_list}{A list of JRIM CDS object generated using \code{\link{make_JRIM_cds}}.}

\item{s}{Power law value. See details for more information.}

\item{maxit}{Maximum number of iterations for distance_parameter estimation.}

\item{weights}{'equal' or 'sample.size'. The selection of n_k in joint graphical lasso model.
when weights='equal', n_k=1 for all (k in 1:K). when weights='sample.size', n_k=the group 
number of tissue k.}

\item{w2_initial}{The initial value of reparameterized parameter w_2.}

\item{window}{Size of the genomic window to query, in base pairs.}

\item{sample_num}{Number of random windows to calculate
\code{distance_parameter} for.}

\item{max_elements}{Maximum number of elements per window allowed. Prevents
very large models from slowing performance.}

\item{penalize.diagonal}{Logical, should the diagonal elements be penalized? See JGL packages for details.}

\item{distance_constraint}{Maximum distance of expected connections. Must be
smaller than \code{window}.}

\item{w1_convergence}{Convergence step size for w_1 calculation.}

\item{w2_convergence}{Convergence step size for w_2 calculation.}

\item{verbose}{Logical, should the function run silently?}

\item{genomic_coords}{Either a data frame or a path (character) to a file
with chromosome lengths. The file should have two columns, the first is
the chromosome name (ex. "chr1") and the second is the chromosome length
in base pairs. See \code{data(human.hg19.genome)} for an example. If a
file, should be tab-separated and without header.}

\item{sparsity_constrain}{The requirement of sparsity in each local window.}

\item{long_dist_constrain}{The requirement of sparsity of long distance interactions in each local window.}

\item{similarity_constrain}{The requirement of similarity across networks in each local window.}
}
\value{
A list of results of length \code{sample_num} and the resulting value of lambda1 lambda2.
}
\description{
Function to calculate scaling parameters lambda1 and lambda2
for random genomic windows.
}
