% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_JRIM_models.R
\name{generate_JRIM_models}
\alias{generate_JRIM_models}
\title{Generate JRIM models in local genomic windows}
\usage{
generate_JRIM_models(cds_list, lambda1, lambda2, weights = "equal",
  s = 0.75, window = 5e+05, maxiter = 500, max_elements = 300,
  tol = 1e-05, verbose = FALSE, penalize.diagonal = FALSE,
  genomic_coords = cicero::human.hg19.genome)
}
\arguments{
\item{cds_list}{JRIM CDS objects of K tissues generated using \code{\link{make_JRIM_cds}}.}

\item{lambda1}{Scaling parameter value of l1 norm penalty. Generally,
the mean of the calculated lambda1 values from
\code{\link{estimate_JRIM_parameters}}.}

\item{lambda2}{Scaling parameter value of group lasso penalty. Generally,
the mean of the calculated lambda2 values from
\code{\link{estimate_JRIM_parameters}}.}

\item{weights}{'equal' or 'sample.size'. The selection of n_k in joint graphical lasso model.
when weights='equal', n_k=1 for all (k in 1:K). when weights='sample.size', n_k=the group 
number of tissue k.}

\item{s}{Power law value of distance penalty.}

\item{window}{Size of the genomic window to query, in base pairs.}

\item{maxiter}{Maximum iteration number of joint graphical lasso model. 
See JGL packages for details.}

\item{max_elements}{Maximum number of elements per window allowed. Prevents
very large models from slowing performance.}

\item{tol}{The convergence parameter of joint graphical lasso model. See JGL packages for details.}

\item{verbose}{Logical, should the function run silently?}

\item{penalize.diagonal}{Logical, should the diagonal elements be penalized? See JGL packages for details.}

\item{genomic_coords}{Either a data frame or a path (character) to a file
with chromosome lengths. The file should have two columns, the first is
the chromosome name (ex. "chr1") and the second is the chromosome length
in base pairs. See \code{data(human.hg19.genome)} for an example. If a
file, should be tab-separated and without header.}
}
\value{
A list of results for each window. This list can be
  directly input into \code{\link{assemble_connections_JRIM}} 
  to create regulatory interaction maps of different cell populatins.
}
\description{
Function to adopt joint graphical lasso models on 
overlapping genomic windows.
}
