% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_multiple_connections.R
\name{plot_multiple_connections}
\alias{plot_multiple_connections}
\title{Plot connections of multiple tissues}
\usage{
plot_multiple_connections(connections_list, K, chr, minbp, maxbp,
  fontsize = 6, coaccess_cutoff = 0, peak_color = "#B4656F",
  connection_color = "#7F7CAF", connection_color_legend = TRUE,
  alpha_by_coaccess = FALSE, connection_width = 2,
  connection_ymax = NULL, gene_model = NULL, gene_model_size = 1,
  gene_model_color = "#81D2C7", gene_model_shape = c("smallArrow",
  "box"), collapseTranscripts = FALSE, include_axis_track = TRUE,
  axis_track_size = 0.2, return_as_list = FALSE, viewpoint = NULL,
  viewpoint_color = "#F0544F", viewpoint_fill = "#EFD8D7",
  viewpoint_alpha = 0.5)
}
\arguments{
\item{connections_list}{Data frame of connections of K tissues, which must include the
columns 'Peak1', 'Peak2', and 'coaccess'.}

\item{K}{The number of tissues.}

\item{chr}{The chromosome of the region you would like to plot in the form
'chr10'.}

\item{minbp}{The base pair coordinate of the start of the region to be
plotted.}

\item{maxbp}{The base pair coordinate of the end of the region to be plotted.}

\item{fontsize}{The size of fontsize.}

\item{coaccess_cutoff}{The minimum co-accessibility score you would
like to be plotted. Default is 0. It can be a number or a list of number in length K.}

\item{peak_color}{Color for peak annotations - a single color, the name of a
column containing color values that correspond to Peak1, or the name of
column containing a character or factor to base peak colors on.}

\item{connection_color}{Color for connection lines. A single color, the name
of a column containing color values, or the name of a column containing a
character or factor to base connection colors on.}

\item{alpha_by_coaccess}{Logical, should the transparency of connection
lines be scaled based on co-accessibility score?}

\item{connection_width}{Width of connection lines.}

\item{connection_ymax}{Connection y-axis height. If \code{NULL}, chosen
automatically. It can be a number or a list of number in length K.}

\item{gene_model}{Either \code{NULL} or a data.frame. The data.frame should
be in a form compatible with the Gviz function
\code{\link[Gviz]{GeneRegionTrack-class}} (cannot have NA as column names).}

\item{gene_model_size}{The size of plotted gene model.}

\item{gene_model_color}{Color for gene annotations.}

\item{gene_model_shape}{Shape for gene models, passed to
\code{\link[Gviz]{GeneRegionTrack-class}}. Options described at
\code{\link[Gviz]{GeneRegionTrack-class}}.}

\item{collapseTranscripts}{Logical or character scalar. Can be one in
\code{gene}, \code{longest}, \code{shortest} or \code{meta}. Variable is
passed to the \code{\link[Gviz]{GeneRegionTrack-class}} function of Gviz.
Determines whether and how to collapse related transcripts. See Gviz
documentation for details.}

\item{include_axis_track}{Logical, should a genomic axis be plotted?}

\item{return_as_list}{Logical, if TRUE, the function will not plot, but will
return the plot components as a list. Allows user to add/customize Gviz
components and plot them separately using \code{\link[Gviz]{plotTracks}}.}

\item{viewpoint}{\code{NULL} or Coordinates in form "chr1_10000_10020". Use
viewpoint if you would like to plot cicero connections "4C-seq style".
Only connections originating in the viewpoint will be shown. Ideal for
comparisons with 4C-seq data. If comparison_viewpoint is \code{TRUE}, any
comparison track will be subsetted as well.}

\item{viewpoint_color}{Color for the highlight border.}

\item{viewpoint_fill}{Color for the highlight fill.}

\item{viewpoint_alpha}{Alpha value for the highlight fill.}

\item{comparison_viewpoint}{Logical, should viewpoint apply to comparison
track as well?}
}
\value{
A gene region plot, or list of components if return_as_list is
  \code{TRUE}.
}
\description{
Plotting function for JRIM connections of multiple tissues. Uses \code{\link[Gviz]{plotTracks}}
as its basis
}
